//Ludum Dare 23
//48-hour competition entry
//"Everlasting"
//April 20~22, 2012
//48-hour game competition
//Author: Benjamin C Watt (@feyleafgames)
//Theme: Tiny World

////////////////////////////////////////////////

#define THE_WORLD_IS_TINY

#include <SFML/Graphics.hpp>
#include <SFML/Audio.hpp>
#include <SFML/Window.hpp>
#include "map.h"
#include "time.h"
#include "math.h"
#include "player.h"
#include "treasure.h"
#include "monster.h"
#include "coin.h"
#include "hud.h"
#include "reader.h"

struct BiomeStruct
{
	sf::Color maincolor;
	sf::Color vibrance;
};

int main(void)
{
	int gamestate=0;
	bool isPause=false;
	sf::SoundBuffer musicBuf;
	sf::SoundBuffer fanfareBuf;
	sf::SoundBuffer diedBuf;
	sf::SoundBuffer chingBuf;
	musicBuf.LoadFromFile("files/mysterious-tiny-world.ogg");
	fanfareBuf.LoadFromFile("files/fanfare.ogg");
	diedBuf.LoadFromFile("files/ohno.ogg");
	chingBuf.LoadFromFile("files/coin.ogg");
	sf::Image title;
	title.LoadFromFile("files/titlescreen.png");
	sf::Sprite titleSprite;
	titleSprite.SetImage(title);
	sf::Sound music;
	sf::Sound fanfare;
	sf::Sound oops;
	sf::Sound ching;
	music.SetBuffer(musicBuf);
	fanfare.SetBuffer(fanfareBuf);
	oops.SetBuffer(diedBuf);
	ching.SetBuffer(chingBuf);
	music.SetLoop(true);
	fanfare.SetLoop(false);
	oops.SetLoop(false);
	ching.SetLoop(false);
	ReaderClass text;
	HUDClass HUD;
	const int maxcoins=64;
	const int maxmonsters=16;
	int numcoins=20;
	int nummonsters=0;
	int coincount=0;
	srand((unsigned)time(NULL));
	sf::Clock testclock;
	testclock.Reset();
	bool quit=false;
	bool soundon=true;

	sf::Font digi;
	digi.LoadFromFile("files/digi.ttf", 32);

	sf::String message;
	message.SetFont(digi);
	message.SetSize(16);
	bool isMessage=false;
	message.SetText("You have died. Press \'R\' to\nlive again in a new biome!");
	sf::String gmessage;
	gmessage.SetFont(digi);
	gmessage.SetSize(16);
	bool isGMessage=false;
	gmessage.SetText("You saved all the tiny hearts!\nYou will proceed to the\nnext world.");
	sf::String intro;
	intro.SetFont(digi);
	intro.SetSize(32);
	intro.SetText("\n\n\n      Everlasting\n\nby Benjamin C. Watt\nFor Ludum Dare 23\n1 = start 2 = readme");

	sf::String hud;
	BiomeStruct volcano;
	volcano.maincolor=sf::Color(186, 86, 50);
	volcano.vibrance=sf::Color(40, 14, 14);
	BiomeStruct snowpeak;
	snowpeak.maincolor=sf::Color(203, 205, 214);
	snowpeak.vibrance=sf::Color(137, 192, 196);
	BiomeStruct forest;
	forest.maincolor=sf::Color(95, 129, 49);
	forest.vibrance=sf::Color(122, 84, 46);
	BiomeStruct meadow;
	meadow.maincolor=sf::Color(144, 196, 15);
	meadow.vibrance=sf::Color(184, 196, 0);

	BiomeStruct ocean;
	ocean.maincolor=sf::Color(25, 124, 221);
	ocean.vibrance=sf::Color(74, 14, 77);
	BiomeStruct island;
	island.maincolor=sf::Color(233, 221, 129);
	island.vibrance=sf::Color(102, 179, 221);
	BiomeStruct desert;
	desert.maincolor=sf::Color(210, 201, 74);
	desert.vibrance=sf::Color(192, 187, 141);
	BiomeStruct castle;
	castle.maincolor=sf::Color(68, 68, 68);
	castle.vibrance=sf::Color(154, 206, 219);

	BiomeStruct biomeList[8];
	biomeList[0]=volcano;
	biomeList[1]=snowpeak;
	biomeList[2]=forest;
	biomeList[3]=meadow;
	biomeList[4]=ocean;
	biomeList[5]=island;
	biomeList[6]=desert;
	biomeList[7]=castle;

	int biome=0;
	MapClass theMap;
	PlayerClass player;
	MonsterClass monster[maxmonsters];
	CoinClass coin[maxcoins];
	for(int i=0; i<nummonsters; i++) monster[i].init(theMap, player);
	for(int i=0; i<numcoins; i++) coin[i].init(theMap);
	TreasureClass trophy;
	trophy.position = theMap.treasurePosition;
	player.position=theMap.getPlayer(theMap.currentRoom);
	sf::RenderWindow app(sf::VideoMode(16*32, 9*32, 32), "Everlasting by Benjamin C. Watt (#LD48)");
	app.SetPosition(80, 40);
	sf::Event theEvent;
	app.SetFramerateLimit(30);
	HUD.init(0);
	HUD.setcoins(0);
	HUD.setbiome(biome);
	music.Play();
	while(!quit)
	{
		if(soundon)
		{
			music.SetVolume(75);
			fanfare.SetVolume(75);
			oops.SetVolume(75);
			ching.SetVolume(75);
		}
		else
		{
			music.SetVolume(0);
			fanfare.SetVolume(0);
			oops.SetVolume(0);
			ching.SetVolume(10);
		}

		while(app.GetEvent(theEvent))
		{
			if(theEvent.Type == sf::Event::Closed) quit=true;
		}

//debug level shuffler
		if((app.GetInput().IsKeyDown(sf::Key::R) && testclock.GetElapsedTime()>0.5f) || (player.cheeranimation && player.cheerclock.GetElapsedTime()>5))
		{
			if(HUD.allCollected()) HUD.cleartrophy();
			if(!(music.GetStatus()==sf::Sound::Playing)) music.Play();
			if(player.deathanimation) {HUD.setcoins(0); HUD.cleartrophy();}
			theMap.init();
			biome=(biome+1)%8;
			HUD.setbiome(biome);
			//HUD.init(theMap.PutTheHUD);
			player.deathanimation=false;
			player.cheeranimation=false;
			nummonsters=rand()%3+2;
			if(HUD.coins>100) nummonsters=10;
			player.position=theMap.getPlayer(theMap.currentRoom);
			trophy.position=theMap.treasurePosition;
			for(int i=0; i<nummonsters; i++) monster[i].init(theMap, player);
			for(int i=0; i<numcoins; i++) coin[i].init(theMap);
			testclock.Reset();
		}
		if(app.GetInput().IsKeyDown(sf::Key::M) && testclock.GetElapsedTime()>0.5f)
		{
			soundon=!soundon;
			testclock.Reset();
		}
		if(app.GetInput().IsKeyDown(sf::Key::Escape) && testclock.GetElapsedTime()>0.5f) {isPause=!isPause; testclock.Reset();}

		//game controls
		if(app.GetInput().IsKeyDown(sf::Key::Up)|| app.GetInput().IsKeyDown(sf::Key::W)) {player.move(sf::Vector2f(0, -2)); testclock.Reset();}
		if(app.GetInput().IsKeyDown(sf::Key::Down)|| app.GetInput().IsKeyDown(sf::Key::S)) {player.move(sf::Vector2f(0, 2)); testclock.Reset();}
		if(app.GetInput().IsKeyDown(sf::Key::Left)|| app.GetInput().IsKeyDown(sf::Key::A)) {player.move(sf::Vector2f(-2, 0)); testclock.Reset();}
		if(app.GetInput().IsKeyDown(sf::Key::Right)|| app.GetInput().IsKeyDown(sf::Key::D)) {player.move(sf::Vector2f(2, 0)); testclock.Reset();}
		if(!(player.position.x+5>trophy.position.x+16 || player.position.x+9<trophy.position.x || player.position.y+4>trophy.position.y+16 || player.position.y+16<trophy.position.y)) {player.cheered(); HUD.settrophy(biome, true); testclock.Reset();}
		app.Clear(biomeList[biome].maincolor);
		theMap.setcolor(biomeList[biome].vibrance);
//		trophy.sprite.SetColor(biomeList[biome].vibrance);
		if(!isPause)
		{
			player.update(theMap);
			for(int i=0; i<nummonsters; i++)
			{
				if(!(monster[i].position.x+5>player.position.x+10 || monster[i].position.x+13<player.position.x+6 || monster[i].position.y+5>player.position.y+13 || monster[i].position.y+13<player.position.y+4)) {if(monster[i].isActive) player.died();}
				monster[i].update(theMap, player);
			}
			for(int i=0; i<numcoins; i++)
			{
				if(coin[i].position==trophy.position) coin[i].isActive=false;
				if((coin[i].isActive) && !(coin[i].position.x>player.position.x+10 || coin[i].position.x+16<player.position.x+6 || coin[i].position.y>player.position.y+13 || coin[i].position.y+16<player.position.y+4)) {coin[i].isActive=false; ching.Play(); HUD.addcoin();}
				coin[i].update();
			}
		}
		message.SetPosition(60, 90);
		gmessage.SetPosition(60, 90);
		theMap.render(app);
		trophy.render(app);
		for(int i=0; i<nummonsters; i++) monster[i].render(app);
		for(int i=0; i<numcoins; i++) coin[i].render(app);
		player.render(app);
		if(player.deathanimation && player.deathclock.GetElapsedTime()>2) isMessage=true; else isMessage=false;
		if(isMessage) app.Draw(message);
		if(player.cheeranimation && player.cheerclock.GetElapsedTime()>2 && HUD.allCollected()) isGMessage=true; else isGMessage=false;
		if(isGMessage) app.Draw(gmessage);

		if(player.deathanimation && player.deathclock.GetElapsedTime()<0.25f) {music.Stop(); oops.Play();}
		if(player.cheeranimation && player.cheerclock.GetElapsedTime()<0.25f) {music.Stop(); fanfare.Play();}

		HUD.render(app);
		if(gamestate<2)
		{
			if(app.GetInput().IsKeyDown(sf::Key::Num1) || app.GetInput().IsKeyDown(sf::Key::Numpad1)) gamestate=2;
			if(app.GetInput().IsKeyDown(sf::Key::Num2) || app.GetInput().IsKeyDown(sf::Key::Numpad2)) gamestate=1;

			app.Draw(titleSprite);
			if(gamestate==1)
			{
				text.update();
				text.render(app);
			}
			else
			{
				app.Draw(intro);
			}
		}


		app.Display();
	}
	return EXIT_SUCCESS;
}

//------------------------
