#include "monster.h"
#include "map.h"

MonsterClass::MonsterClass()
{
	position.x=0;
	position.y=0;
	facing=2;
	isActive=true;
	image.LoadFromFile("files/sprites.png");
	image.SetSmooth(false);
	sprite.SetImage(image);
	sprite.SetSubRect(sf::IntRect(facing*16, 48, facing*16+16, 64));
	sprite.SetPosition(position);
}

void MonsterClass::init(MapClass& map, PlayerClass& pl)
{
	if(!pl.deathanimation)
	{
	int attempts=0;
	isActive=true;
	int rx=rand()%32;
	int ry=rand()%18;
	position=sf::Vector2f(float(rx*16), float(ry*16));
	float distance=sqrt(((pl.position.x-position.x)*(pl.position.x-position.x)) + ((pl.position.y-position.y)*(pl.position.y-position.y)));
	while((map.tile[rx][ry].wall || (distance<150)) && attempts<100)
	{
		attempts++;
		rx=rand()%32;
		ry=rand()%18;
		position=sf::Vector2f(float(rx*16), float(ry*16));
		distance=sqrt(((pl.position.x-position.x)*(pl.position.x-position.x)) + ((pl.position.y-position.y)*(pl.position.y-position.y)));
	}
	if(attempts>99) isActive=false;
	position=sf::Vector2f(float(rx*16), float(ry*16));
	sprite.SetPosition(position);
	velocity=sf::Vector2f(0, 0);
	}
}

void MonsterClass::render(sf::RenderWindow &win)
{

	if(isActive)
	{
		sprite.SetSubRect(sf::IntRect(facing*16, 48, facing*16+16, 64));
		sprite.SetPosition(position);
		win.Draw(sprite);
	}
}

void MonsterClass::move(sf::Vector2f vel)
{
	velocity+=vel;
}

void MonsterClass::update(MapClass& map, PlayerClass& pl)
{
	if(!pl.deathanimation && !pl.cheeranimation)
	{
	if(isActive)
	{

		float distance=sqrt(((pl.position.x-position.x)*(pl.position.x-position.x)) + ((pl.position.y-position.y)*(pl.position.y-position.y)));
		velocity=(pl.position-position)/distance;
		if(velocity.x>=0) facing=1;
		if(velocity.x<=0) facing=3;
		if(velocity.y>=0 && velocity.y>=velocity.x) facing=2;
		if(velocity.y<=0 && velocity.y<=velocity.x) facing=0;

		position+=velocity;
		int tx=int((position.x+8)/16);
		int ty=int((position.y+10)/16);
		if(map.tile[tx][ty].wall)
		{
			position-=velocity;
			velocity=sf::Vector2f(0, 0);
		}

		if(velocity.x<0.014 || velocity.x> -0.014) velocity.x=0;
		else velocity.x/=1.00618f;
		if(velocity.y<0.014 || velocity.y> -0.014) velocity.y=0;
		else velocity.y/=1.00618f;

		sprite.SetSubRect(sf::IntRect(facing*16, 48, facing*16+16, 64));
		sprite.SetPosition(position);
	}
	}
}