#include "player.h"
#include "map.h"

PlayerClass::PlayerClass()
{
	position.x=0;
	position.y=0;
	deathanimation=false;
	cheeranimation=false;
	facing=1;//east
	image.LoadFromFile("files/sprites.png");
	image.SetSmooth(false);
	sprite.SetImage(image);
	sprite.SetSubRect(sf::IntRect(facing*16, 32, facing*16+16, 48));
	sprite.SetPosition(position);
}

void PlayerClass::render(sf::RenderWindow &win)
{
	int spot=0;
	if(!deathanimation && !cheeranimation)
	{
		sprite.SetSubRect(sf::IntRect(facing*16, 32, facing*16+16, 48));
	}
	else if(deathanimation)
	{
		spot=int(deathclock.GetElapsedTime()*2);
		if(spot>3) spot=3;
		sprite.SetSubRect(sf::IntRect(spot*16, 16, spot*16+16, 32));
	}
	else if(cheeranimation)
	{
		spot=int(cheerclock.GetElapsedTime()*2);
		if(spot>1) spot=1;
		sprite.SetSubRect(sf::IntRect(spot*16+32, 0, spot*16+48, 16));
	}
	sprite.SetPosition(position);
	win.Draw(sprite);

}

void PlayerClass::move(sf::Vector2f vel)
{
	if(vel.x>0) facing=1;
	else if(vel.x<0) facing=3;
	if(vel.y>0) facing=2;
	else if(vel.y<0) facing=0;
	velocity+=vel;
}

void PlayerClass::update(MapClass& map)
{
	if(!deathanimation && !cheeranimation)
	{
		position+=velocity;
		int tx=int((position.x+8)/16);
		int ty=int((position.y+8)/16);
		if(map.tile[tx][ty].wall)
		{
			position-=velocity;
		}

		if(velocity.x<0.014 || velocity.x> -0.014) velocity.x=0;
		else velocity.x/=1.00618f;
		if(velocity.y<0.014 || velocity.y> -0.014) velocity.y=0;
		else velocity.y/=1.00618f;

		sprite.SetPosition(position);
		if(velocity.x>0 && velocity.x>=velocity.y) facing=1;
		if(velocity.x<0 && velocity.x>=velocity.y) facing=3;
		if(velocity.y>0 && velocity.y>=velocity.x) facing=2;
		if(velocity.y<0 && velocity.y>=velocity.x) facing=0;
	}

}

void PlayerClass::died()
{
	if(!deathanimation)
	{
		deathanimation=true;
		deathclock.Reset();
	}
}

void PlayerClass::cheered()
{
	if(!cheeranimation)
	{
		cheeranimation=true;
		cheerclock.Reset();
	}
}